@@ -40,10 +40,10 @@
 #'
 #' \itemize{
 #' \item \code{strength}, the strength parameter, or total mass, of the marginal Dirichlet processes (default 1);
-#' \item \code{m0}, mean of the normal base measure on the location parameter (default 0);
+#' \item \code{m0}, mean of the normal base measure on the location parameter (default is the sample mean of the data);
 #' \item \code{k0}, scale factor appearing in the normal base measure on the location parameter (default 1);
 #' \item \code{a0}, shape parameter of the inverse gamma base measure on the scale parameter (default 2);
-#' \item \code{b0}, scale parameter of the inverse gamma base measure on the scale parameter (default 1);
+#' \item \code{b0}, scale parameter of the inverse gamma base measure on the scale parameter (default is the sample variance of the data);
 #' \item \code{wei}, parameter controlling the strength of dependence across Dirichlet processes (default 1/2).
 #' }
 #'
@@ -163,10 +163,10 @@ DDPdensity <- function(y,
   group <- as.numeric(as.factor(group))
   ngr   <- length(unique(group))
 
-  if(is.null(prior$m0)){ m0 = 0 } else { m0 = prior$m0 }
+  if(is.null(prior$m0)){ m0 = mean(y) } else { m0 = prior$m0 }
   if(is.null(prior$k0)){ k0 = 1 } else { k0 = prior$k0 }
   if(is.null(prior$a0)){ a0 = 2 } else { a0 = prior$a0 }
-  if(is.null(prior$b0)){ b0 = 1 } else { b0 = prior$b0 }
+  if(is.null(prior$b0)){ b0 = var(y) } else { b0 = prior$b0 }
 
   if(is.null(prior$strength)){ strength = 1 } else { strength = prior$strength }
   if(is.null(prior$wei)){ wei = 0.5 } else { wei = prior$wei }
